from .translation_job_status import TranslationJobStatus
from .translation_model import TranslationModel


class TranslationLanguageOptions:
    """Translation language request options."""
    def __init__(
            self,
            language: str = None,
            model: TranslationModel = None):
        self.language = language
        self.model = model

    def to_dict(self):
        """Returns the raw form of the url data object as the api
        expects them"""
        dict_result = {}
        if self.language:
            dict_result['language'] = self.language
        if self.model:
            dict_result['model'] = self.model

        return dict_result


class TranslationLanguage(TranslationLanguageOptions):
    """Translation language options."""
    def __init__(
            self,
            language: str = None,
            model: TranslationModel = None,
            status: TranslationJobStatus = None,
            failure: str = None):
        super().__init__(language, model)
        self.status = status
        self.failure = failure

    @classmethod
    def from_json(cls, json):
        if json is None:
            return None
        return cls(
            json.get('language'),
            json.get('model'),
            json.get('status'),
            json.get('failure')
        )



import pytest

from src.rev_ai.apiclient import RevAiAPIClient
from src.rev_ai.models import RevAiApiDeploymentConfigMap, RevAiApiDeployment

try:
    from urllib.parse import urljoin
except ImportError:
    from urlparse import urljoin

TOKEN = "token"
JOB_ID = '1'
SPEECH_TO_TEXT_URL = f"{RevAiApiDeploymentConfigMap[RevAiApiDeployment.US]['base_url']}/speechtotext/v1/"
JOB_ID_URL = urljoin(SPEECH_TO_TEXT_URL, 'jobs/{}'.format(JOB_ID))
JOBS_URL = urljoin(SPEECH_TO_TEXT_URL, 'jobs')


@pytest.mark.usefixtures('mock_session', 'make_mock_response')
class TestAsyncCaptionsTranslation():

    def test_get_translated_captions(self, mock_session, make_mock_response):
        client = RevAiAPIClient(TOKEN)
        url = '{}/captions/translation/{}'.format(JOB_ID_URL, "es")
        response = make_mock_response(url=url, text='es captions')
        mock_session.request.return_value = response

        captions = client.get_translated_captions(JOB_ID, "es")
        assert captions == 'es captions'
        mock_session.request.assert_called_once_with(
            "GET",
            url,
            headers=self.hdr_fixture(client, {'Accept': 'application/x-subrip'}))

    def test_get_translated_captions_as_stream(self, mock_session, make_mock_response):
        client = RevAiAPIClient(TOKEN)
        url = '{}/captions/translation/{}'.format(JOB_ID_URL, "es")
        response = make_mock_response(url=url, text='es captions')
        mock_session.request.return_value = response

        captions = client.get_translated_captions_as_stream(JOB_ID, "es")
        assert captions.content == 'es captions'
        mock_session.request.assert_called_once_with(
            "GET",
            url,
            headers=self.hdr_fixture(client, {'Accept': 'application/x-subrip'}), stream=True)

    @staticmethod
    def hdr_fixture(client: RevAiAPIClient, additional_headers):
        hdr = {}
        hdr.update(client.default_headers)
        hdr.update(additional_headers)
        return hdr



# -*- coding: utf-8 -*-
"""Topic extraction result model"""


class TopicExtractionResult:
    def __init__(self, topics):
        """
        :param topics: list of topics included in output
        """
        self.topics = topics

    def __eq__(self, other):
        """Override default equality operator"""
        if isinstance(other, self.__class__):
            return all(a == b for a, b in zip(self.topics, other.topics))
        return False

    @classmethod
    def from_json(cls, json):
        """Alternate constructor used for parsing json"""
        return cls([Topic.from_json(topic) for topic in json.get('topics', [])])


class Topic:
    def __init__(self, topic_name, score, informants):
        """
        :param topic_name: name of the topic, pulled directly from somewhere in the input text
        :param score: score of the topic, between 0 and 1. Higher means it is more likely that this
                      is truly a topic
        :param informants: pieces of the input text which informed this choice of topic
        """
        self.topic_name = topic_name
        self.score = score
        self.informants = informants

    def __eq__(self, other):
        """Override default equality operator"""
        if isinstance(other, self.__class__):
            return all(a == b for a, b in zip(self.informants, other.informants)) \
                and self.topic_name == other.topic_name \
                and self.score == other.score
        return False

    @classmethod
    def from_json(cls, json):
        """Alternate constructor used for parsing json"""
        return cls(
            json['topic_name'],
            json['score'],
            [Informant.from_json(informant) for informant in json.get('informants', [])])


class Informant:
    def __init__(self, content, timestamp=None, end_timestamp=None, offset=None, length=None):
        """
        :param content: content of the informant, pulled from input
        :param timestamp: time at which this element starts if input was json
        :param end_timestamp: time at which this element ends if input was json
        :param offset: Character index at which the content started in the source transcript,
                       excludes invisible characters
        :param length: Length of the content in characters, excludes invisible characters
        """
        self.content = content
        self.timestamp = timestamp
        self.end_timestamp = end_timestamp
        self.offset = offset
        self.length = length

    def __eq__(self, other):
        """Override default equality operator"""
        if isinstance(other, self.__class__):
            return self.__dict__ == other.__dict__
        return False

    @classmethod
    def from_json(cls, json):
        """Alternate constructor used for parsing json"""
        return cls(
            json['content'],
            json.get('ts'),
            json.get('end_ts'),
            json.get('offset'),
            json.get('length'))



# -*- coding: utf-8 -*-
"""Generic client used to interact with our newer style apis"""

from .baseclient import BaseClient
from .models import RevAiApiDeploymentConfigMap, RevAiApiDeployment

try:
    from urllib.parse import urljoin
except ImportError:
    from urlparse import urljoin


class GenericApiClient(BaseClient):
    """Generic client which handles logic for making requests to almost any Rev AI Api.
    Intended to be inherited and extended by a specific client per API"""

    # Default url for US Rev AI deployment
    default_url = RevAiApiDeploymentConfigMap[RevAiApiDeployment.US]['base_url']

    def __init__(
        self,
        access_token: str,
        api_name: str,
        api_version: str,
        parse_job_info,
        parse_job_result,
        url: str = None
    ):
        """Constructor

        :param access_token: access token which authorizes all requests and links them to your
                             account. Generated on the settings page of your account dashboard
                             on Rev AI.
        :param api_name: name of the api to submit to
        :param api_version: version of the api to submit to
        :param parse_job_info: method to be used to parse job information
        :param parse_job_result: method to be used to parse job results
        :param url (optional): url of the Rev AI API deployment to use, defaults to the US
                    deployement, i.e. 'https://api.rev.ai', which can be referenced as
                    RevAiApiDeploymentConfigMap[RevAiApiDeployment.US]['base_url'].
        """

        BaseClient.__init__(self, access_token)
        self.base_url = f'{url if url else self.default_url}/{api_name}/{api_version}/'
        self.parse_job_info = parse_job_info
        self.parse_job_result = parse_job_result

    def _submit_job(self, payload):
        """Submit a job to the api. This method is special in that it is intended to be hidden by
        the implementation this is done because python standard is to pass options individually
        instead of as an object and our true clients should match this standard

        :param payload: payload to be sent with job request
        :raises: HTTPError
        """
        response = self._make_http_request(
            "POST",
            urljoin(self.base_url, 'jobs'),
            json=payload
        )

        return self.parse_job_info(response.json())

    def get_job_details(self, id_):
        """View information about a specific job.
        The server will respond with the status and creation date.

        :param id_: id of the job to be requested
        :returns: Job info object
        :raises: HTTPError
        """
        if not id_:
            raise ValueError('id_ must be provided')

        response = self._make_http_request(
            "GET",
            urljoin(self.base_url, 'jobs/{}'.format(id_))
        )

        return self.parse_job_info(response.json())

    def get_list_of_jobs(self, limit=None, starting_after=None):
        """Get a list of jobs submitted within the last 30 days in reverse
        chronological order up to the provided limit number of jobs per call.
        Pagination is supported via passing the last job id from previous call into starting_after.

        :param limit: optional, limits the number of jobs returned,
                      if none, a default of 100 jobs is returned, max limit if 1000
        :param starting_after: optional, returns jobs created after the job with this id,
                               exclusive (job with this id is not included)
        :returns: list of jobs response data
        :raises: HTTPError
        """
        params = []
        if limit is not None:
            params.append('limit={}'.format(limit))
        if starting_after is not None:
            params.append('starting_after={}'.format(starting_after))

        query = '?{}'.format('&'.join(params))
        response = self._make_http_request(
            "GET",
            urljoin(self.base_url, 'jobs{}'.format(query))
        )

        return [self.parse_job_info(job) for job in response.json()]

    def _get_result_json(self, id_, params):
        """Get the result of a job. This method is special in that it is intended to be hidden by
        the implementation this is done because python standard is to pass options individually
        instead of as an object and our true clients should match this standard

        :param id_: id of job to be requested
        :returns: job result data as raw json
        :raises: HTTPError
        """
        if not id_:
            raise ValueError('id_ must be provided')

        query_params = []
        for key, value in params.items():
            if value is not None:
                query_params.append('{0}={1}'.format(key, value))

        response = self._make_http_request(
            "GET",
            urljoin(self.base_url, 'jobs/{0}/result?{1}'.format(id_, '&'.join(query_params)))
        )

        return response.json()

    def _get_result_object(self, id_, params):
        """Get the result of a job. This method is special in that it is intended to be hidden by
        the implementation this is done because python standard is to pass options individually
        instead of as an object and our true clients should match this standard

        :param id_: id of job to be requested
        :returns: job result data as object
        :raises: HTTPError
        """
        return self.parse_job_result(self._get_result_json(id_, params))

    def delete_job(self, id_):
        """Delete a specific job
        All data related to the job, such as input media and result, will be permanently
        deleted. A job can only by deleted once it's completed.

        :param id_: id of job to be deleted
        :returns: None if job was successfully deleted
        :raises: HTTPError
        """
        if not id_:
            raise ValueError('id_ must be provided')

        self._make_http_request(
            "DELETE",
            urljoin(self.base_url, 'jobs/{}'.format(id_)),
        )

        return

    def create_payload_with_source(self, media_url, source_config, metadata, callback_url,
                                   delete_after_seconds, notification_config):
        payload = {}
        if media_url:
            payload['media_url'] = media_url
        if source_config:
            payload['source_config'] = source_config.to_dict()
        self._copy_options(payload, metadata, callback_url, delete_after_seconds,
                           notification_config)
        return payload

    def _enhance_payload(self, payload, metadata, callback_url, delete_after_seconds,
                         notification_config):
        enhanced = payload.copy()
        self._copy_options(enhanced, metadata, callback_url, delete_after_seconds,
                           notification_config)
        return enhanced

    @staticmethod
    def _copy_options(payload, metadata, callback_url, delete_after_seconds,
                      notification_config):
        if metadata:
            payload['metadata'] = metadata
        if callback_url:
            payload['callback_url'] = callback_url
        if delete_after_seconds is not None:
            payload['delete_after_seconds'] = delete_after_seconds
        if notification_config:
            payload['notification_config'] = notification_config.to_dict()



from enum import Enum
from typing import Dict, TypedDict


class RevAiApiDeployment(Enum):
    """
    Enum representing the deployment regions for Rev AI API.

    Attributes:
        US: Represents the United States deployment.
        EU: Represents the European Union deployment.
    """
    US = 'US'
    EU = 'EU'


class RevAiApiConfig(TypedDict):
    """
    TypedDict representing the configuration for a Rev AI API deployment.

    Attributes:
        base_url: The base URL for API requests.
        base_websocket_url: The base URL for WebSocket connections.
    """
    base_url: str
    base_websocket_url: str


# Dictionary mapping RevAiApiDeployment enum values to their respective configuration settings.
RevAiApiDeploymentConfigMap: Dict[RevAiApiDeployment, RevAiApiConfig] = {
    RevAiApiDeployment.US: {
        'base_url': 'https://api.rev.ai',
        'base_websocket_url': 'wss://api.rev.ai'
    },
    RevAiApiDeployment.EU: {
        'base_url': 'https://ec1.api.rev.ai',
        'base_websocket_url': 'wss://ec1.api.rev.ai'
    }
}



# -*- coding: utf-8 -*-
"""Client used or interacting with out sentiment analysis api"""

from .generic_api_client import GenericApiClient
from .models import TopicExtractionJob, TopicExtractionResult


class TopicExtractionClient(GenericApiClient):
    """Client for interacting with the Rev AI topic extraction api"""

    # Default version of Rev AI topic extraction api
    api_version = 'v1'

    # Default api name of Rev AI topic extraction api
    api_name = 'topic_extraction'

    def __init__(self, access_token):
        """Constructor

        :param access_token: access token which authorizes all requests and links them to your
                             account. Generated on the settings page of your account dashboard
                             on Rev AI.
        """

        GenericApiClient.__init__(self, access_token, self.api_name, self.api_version,
                                  TopicExtractionJob.from_json, TopicExtractionResult.from_json)

    def submit_job_from_text(self,
                             text=None,
                             metadata=None,
                             callback_url=None,
                             delete_after_seconds=None,
                             language=None,
                             notification_config=None):
        """Submit a job to the Rev AI topic extraction api. Takes either a plain text string or
        Transcript object
        :param text: Plain text string to be run through topic extraction
        :param metadata: info to associate with the transcription job
        :param callback_url: the callback url to invoke on job completion as a webhook
            .. deprecated:: 2.16.0
                Use notification_config instead
        :param delete_after_seconds: number of seconds after job completion when job is auto-deleted
        :param language: specify language using the one of the supported ISO 639-1 (2-letter) or
            ISO 639-3 (3-letter) language codes as defined in the API Reference
        :param notification_config: CustomerUrlData object containing the callback url to
            invoke on job completion as a webhook and optional authentication headers to use when
            calling the callback url
        :returns: TopicExtractionJob object
        :raises: HTTPError
        """
        payload = self._enhance_payload({'text': text, 'language': language}, metadata,
                                        callback_url, delete_after_seconds, notification_config)
        return self._submit_job(payload)

    def submit_job_from_transcript(self,
                                   transcript=None,
                                   metadata=None,
                                   callback_url=None,
                                   delete_after_seconds=None,
                                   language=None,
                                   notification_config=None):
        """Submit a job to the Rev AI topic extraction api. Takes either a plain text string or
        Transcript object

        :param transcript: Transcript object from the Rev AI async transcription client to be run
                           through topic extraction
        :param metadata: info to associate with the transcription job
        :param callback_url: the callback url to invoke on job completion as a webhook
        .. deprecated:: 2.16.0
            Use notification_config instead
        :param delete_after_seconds: number of seconds after job completion when job is auto-deleted
        :param language: specify language using the one of the supported ISO 639-1 (2-letter) or
            ISO 639-3 (3-letter) language codes as defined in the API Reference
        :param notification_config: CustomerUrlData object containing the callback url to
            invoke on job completion as a webhook and optional authentication headers to use when
            calling the callback url
        :returns: TopicExtractionJob object
        :raises: HTTPError
        """
        payload = self._enhance_payload({'json': transcript.to_dict(), 'language': language},
                                        metadata, callback_url, delete_after_seconds,
                                        notification_config)
        return self._submit_job(payload)

    def get_result_json(self, id_, threshold=None):
        """Get result of a topic extraction job as json.

        :param id_: id of job to be requested
        :param threshold: score threshold for topics. No topics with scores under this threshold
                          will be returned
        :returns: job result data as raw json
        :raises: HTTPError
        """
        return self._get_result_json(id_, {'threshold': threshold})

    def get_result_object(self, id_, threshold=None):
        """Get result of a topic extraction job as TopicExtractionResult object.

        :param id_: id of job to be requested
        :param threshold: score threshold for topics. No topics with scores under this threshold
                          will be returned
        :returns: job result data as TopicExtractionResult object
        :raises: HTTPError
        """
        return self._get_result_object(id_, {'threshold': threshold})



import json

import pytest

from src.rev_ai.apiclient import RevAiAPIClient
from src.rev_ai.models import RevAiApiDeploymentConfigMap, RevAiApiDeployment
from src.rev_ai.models.asynchronous.translation_job_status import TranslationJobStatus
from src.rev_ai.models.asynchronous.translation_language_options import TranslationLanguageOptions
from src.rev_ai.models.asynchronous.translation_options import TranslationOptions
from src.rev_ai.models.asynchronous.translation_model import TranslationModel

try:
    from urllib.parse import urljoin
except ImportError:
    from urlparse import urljoin

TOKEN = "token"
JOB_ID = '1'
SPEECH_TO_TEXT_URL = f"{RevAiApiDeploymentConfigMap[RevAiApiDeployment.US]['base_url']}/speechtotext/v1/"
JOB_ID_URL = urljoin(SPEECH_TO_TEXT_URL, 'jobs/{}'.format(JOB_ID))
JOBS_URL = urljoin(SPEECH_TO_TEXT_URL, 'jobs')


@pytest.mark.usefixtures('mock_session', 'make_mock_response')
class TestAsyncTranslation():

    def test_submit_local_file(self, mocker, mock_session, make_mock_response):
        created_on = '2018-05-05T23:23:22.30Z'
        completed_on = '2018-05-05T23:24:22.30Z'
        status = 'transcribed'
        data = {
            'id': JOB_ID,
            'created_on': created_on,
            'status': status,
            'translation': {
                'target_languages': [
                    {
                        'language': 'es',
                        'model': 'premium',
                        'status': 'completed'
                    },
                    {
                        'language': 'ru',
                        'model': 'premium',
                        'status': 'completed'
                    }
                ],
                'completed_on': completed_on
            }
        }
        response = make_mock_response(url=JOB_ID_URL, json_data=data)
        mock_session.request.return_value = response
        client = RevAiAPIClient(TOKEN)

        with mocker.patch('src.rev_ai.apiclient.open', create=True)() as file:
            job = client.submit_job_local_file('test_mp3.mp3',
                                               language="en",
                                               translation_config=TranslationOptions(
                                                   target_languages=[
                                                       TranslationLanguageOptions("es", TranslationModel.PREMIUM),
                                                       TranslationLanguageOptions("ru")
                                                   ]
                                               ))
        mock_session.request.assert_called_once_with(
            "POST",
            JOBS_URL,
            files={
                'media': ('test_mp3.mp3', file),
                'options': (
                    None,
                    json.dumps({
                        'language': 'en',
                        'translation_config': {
                            'target_languages': [
                                {
                                    'language': 'es',
                                    'model': 'premium'
                                },
                                {
                                    'language': 'ru'
                                }

                            ]}
                    }, sort_keys=True)
                )
            },
            headers=client.default_headers
        )
        assert job.translation is not None
        assert job.translation.target_languages is not None
        assert len(job.translation.target_languages) == 2

        assert job.translation.completed_on is not None
        assert job.translation.target_languages[0].status == TranslationJobStatus.COMPLETED
        assert job.translation.target_languages[0].language == "es"
        assert job.translation.target_languages[0].model == TranslationModel.PREMIUM

        assert job.translation.target_languages[1].status, TranslationJobStatus.COMPLETED
        assert job.translation.target_languages[1].language, "ru"

    def test_submit_source_url(self, mock_session, make_mock_response):
        status = 'transcribed'
        created_on = '2018-05-05T23:23:22.29Z'
        completed_on = '2018-05-05T23:23:22.30Z'
        data = {
            'id': JOB_ID,
            'status': status,
            'created_on': created_on,
            'summarization': {
                'prompt': 'Try to summarize this transcript as good as you possibly can',
                'model': 'premium',
                'type': 'bullets',
                'status': 'completed',
                'completed_on': completed_on
            },
            'translation': {
                'target_languages': [
                    {
                        'language': 'es',
                        'model': 'premium',
                        'status': 'completed'
                    },
                    {
                        'language': 'ru',
                        'model': 'premium',
                        'status': 'completed'
                    }
                ],
                'completed_on': completed_on
            }
        }
        response = make_mock_response(url=JOB_ID_URL, json_data=data)
        mock_session.request.return_value = response

        client = RevAiAPIClient(TOKEN)

        job = client.submit_job_url('https://example.com/test.mp3',
                                    language="en",
                                    translation_config=TranslationOptions(
                                        target_languages=[
                                            TranslationLanguageOptions("es", TranslationModel.PREMIUM),
                                            TranslationLanguageOptions("ru")
                                        ]
                                    )
                                    )
        mock_session.request.assert_called_once_with(
            "POST",
            JOBS_URL,
            json={
                'media_url': 'https://example.com/test.mp3',
                'language': 'en',
                'translation_config': {
                    'target_languages': [
                        {
                            'language': 'es',
                            'model': 'premium'
                        },
                        {
                            'language': 'ru'
                        }

                    ]}
            },
            headers=client.default_headers
        )
        assert job.translation is not None
        assert job.translation.target_languages is not None
        assert len(job.translation.target_languages) == 2

        assert job.translation.completed_on is not None
        assert job.translation.target_languages[0].status == TranslationJobStatus.COMPLETED
        assert job.translation.target_languages[0].language == "es"
        assert job.translation.target_languages[0].model == TranslationModel.PREMIUM

        assert job.translation.target_languages[1].status, TranslationJobStatus.COMPLETED
        assert job.translation.target_languages[1].language, "ru"

    def test_get_translated_transcript_text(self, mock_session, make_mock_response):
        client = RevAiAPIClient(TOKEN)
        url = '{}/transcript/translation/{}'.format(JOB_ID_URL, "es")
        response = make_mock_response(url=url, text='es transcript')
        mock_session.request.return_value = response

        translation = client.get_translated_transcript_text(JOB_ID, "es")
        assert translation == 'es transcript'
        mock_session.request.assert_called_once_with(
            "GET",
            url,
            headers=self.hdr_fixture(client, {'Accept': 'text/plain'}))

    def test_get_translated_transcript_json(self, mock_session, make_mock_response):
        client = RevAiAPIClient(TOKEN)
        url = '{}/transcript/translation/{}'.format(JOB_ID_URL, "es")
        data = {
            'monologues': []
        }
        response = make_mock_response(url=url, json_data=data)
        mock_session.request.return_value = response

        translation = client.get_translated_transcript_json(JOB_ID, "es")
        assert translation == data
        mock_session.request.assert_called_once_with(
            "GET",
            url,
            headers=self.hdr_fixture(client, {'Accept': 'application/vnd.rev.transcript.v1.0+json'}))

    def test_get_translated_transcript_json_as_stream(self, mock_session, make_mock_response):
        client = RevAiAPIClient(TOKEN)
        url = '{}/transcript/translation/{}'.format(JOB_ID_URL, "es")
        data = {
            'monologues': []
        }
        response = make_mock_response(url=url, json_data=data)
        mock_session.request.return_value = response

        translation = client.get_translated_transcript_json_as_stream(JOB_ID, "es")
        s = translation.content.decode('utf-8')
        s = s.replace('\'', '"')
        assert json.loads(s) == data
        mock_session.request.assert_called_once_with(
            "GET",
            url,
            headers=self.hdr_fixture(client, {'Accept': 'application/vnd.rev.transcript.v1.0+json'}), stream=True)

    def test_get_translated_transcript_object(self, mock_session, make_mock_response):
        client = RevAiAPIClient(TOKEN)
        url = '{}/transcript/translation/{}'.format(JOB_ID_URL, "es")
        data = {
            'monologues': [
                {'speaker': 123}
            ]
        }
        response = make_mock_response(url=url, json_data=data)
        mock_session.request.return_value = response

        translation = client.get_translated_transcript_object(JOB_ID, "es")
        assert translation.monologues[0].speaker == 123
        mock_session.request.assert_called_once_with(
            "GET",
            url,
            headers=self.hdr_fixture(client, {'Accept': 'application/vnd.rev.transcript.v1.0+json'}))

    @staticmethod
    def hdr_fixture(client: RevAiAPIClient, additional_headers):
        hdr = {}
        hdr.update(client.default_headers)
        hdr.update(additional_headers)
        return hdr



# -*- coding: utf-8 -*-
"""Unit tests for RevAiApiClient"""

import pytest
import sys

from src.rev_ai.models.customer_url_data import CustomerUrlData
from src.rev_ai.generic_api_client import GenericApiClient
from src.rev_ai import __version__
from tests.helpers import Matcher

try:
    from urllib.parse import urljoin
except ImportError:
    from urlparse import urljoin

TOKEN = 'token'
VERSION = 'version'
API = 'api'
JOB_ID = '1'
METADATA = 'test'
NOTIFICATION_URL = 'https://example.com/'
NOTIFICATION_AUTH = 'headers'
CREATED_ON = '2018-05-05T23:23:22.29Z'
LANGUAGE = 'en'

NOTIFICATION_CONFIG = CustomerUrlData(NOTIFICATION_URL, NOTIFICATION_AUTH)


class TestGenericApiClient:
    def test_constructor_with_success(self):
        client = _create_client()

        headers = client.default_headers

        assert headers.get('User-Agent') == 'RevAi-PythonSDK/{}'.format(__version__)
        assert headers.get('Authorization') == 'Bearer {}'.format(TOKEN)
        assert client.base_url == 'https://api.rev.ai/{0}/{1}/'.format(API, VERSION)

    @pytest.mark.parametrize('token', [None, ''])
    def test_constructor_with_no_token(self, token):
        with pytest.raises(ValueError, match='access_token must be provided'):
            GenericApiClient(token, "v1", "api", lambda x: x, lambda x: x)

    def test_delete_job_success(self, mock_session, make_mock_response):
        client = _create_client()
        url = urljoin(client.base_url, 'jobs/{}'.format(JOB_ID))
        response = make_mock_response(url=url, status=204)
        mock_session.request.return_value = response

        res = client.delete_job(JOB_ID)

        assert res is None
        mock_session.request.assert_called_once_with("DELETE",
                                                     url,
                                                     headers=client.default_headers)

    @pytest.mark.parametrize('id', [None, ''])
    def test_delete_job_with_no_id(self, id, mock_session):
        with pytest.raises(ValueError, match='id_ must be provided'):
            _create_client().delete_job(id)

    @pytest.mark.usefixtures('mock_session', 'make_mock_response')
    def test_get_job_details_with_success(self, mock_session, make_mock_response):
        client = _create_client()
        url = urljoin(client.base_url, 'jobs/{}'.format(JOB_ID))
        status = 'completed'
        created_on = '2018-05-05T23:23:22.29Z'
        data = {
            'id': JOB_ID,
            'status': status,
            'created_on': created_on
        }
        response = make_mock_response(url=url, json_data=data)
        mock_session.request.return_value = response

        res = client.get_job_details(JOB_ID)

        assert res == data
        mock_session.request.assert_called_once_with("GET",
                                                     url,
                                                     headers=client.default_headers)

    @pytest.mark.parametrize('id', [None, ''])
    def test_get_job_details_with_no_job_id(self, id, mock_session):
        with pytest.raises(ValueError, match='id_ must be provided'):
            _create_client().get_job_details(id)

    def test_get_list_of_jobs_limit_with_success(self, mock_session, make_mock_response):
        client = _create_client()
        url = urljoin(client.base_url, 'jobs?limit=2')
        status = 'completed'
        created_on = '2018-05-05T23:23:22.29Z'
        data = [
            {
                'id': JOB_ID,
                'status': status,
                'created_on': created_on
            },
            {
                'id': '2',
                'status': 'in_progress',
                'created_on': created_on
            }
        ]
        response = make_mock_response(url=url, json_data=data)
        mock_session.request.return_value = response

        res = client.get_list_of_jobs(limit=2)

        assert isinstance(res, list)
        assert len(res) == 2
        mock_session.request.assert_called_once_with("GET", url, headers=client.default_headers)

    def test_get_list_of_jobs_starting_after_with_success(self, mock_session, make_mock_response):
        client = _create_client()
        url = urljoin(client.base_url, 'jobs?starting_after=4')
        status = 'completed'
        created_on = '2018-05-05T23:23:22.29Z'
        data = [
            {
                'id': JOB_ID,
                'status': status,
                'created_on': created_on
            }
        ]
        response = make_mock_response(url=url, json_data=data)
        mock_session.request.return_value = response

        res = client.get_list_of_jobs(starting_after="4")

        assert isinstance(res, list)
        assert len(res) == 1
        mock_session.request.assert_called_once_with("GET", url, headers=client.default_headers)

    def test_submit_job_empty_payload_with_success(self, mock_session, make_mock_response):
        client = _create_client()
        url = urljoin(client.base_url, 'jobs')
        data = {
            'id': JOB_ID,
            'status': 'in_progress',
            'delete_after_seconds': 0
        }
        response = make_mock_response(url=url, json_data=data)
        mock_session.request.return_value = response

        res = client._submit_job({})

        assert res == data
        mock_session.request.assert_called_once_with(
            "POST",
            url,
            json={},
            headers=client.default_headers)

    def test_submit_job_nonempty_payload_with_success(self, mock_session, make_mock_response):
        client = _create_client()
        url = urljoin(client.base_url, 'jobs')
        data = {
            'id': JOB_ID,
            'status': 'in_progress',
            'delete_after_seconds': 0
        }
        response = make_mock_response(url=url, json_data=data)
        mock_session.request.return_value = response
        payload = client._enhance_payload({'random': True}, METADATA, None, 0, NOTIFICATION_CONFIG)

        res = client._submit_job(payload)

        assert res == data
        mock_session.request.assert_called_once_with(
            "POST",
            url,
            json=payload,
            headers=client.default_headers)

    def test_get_result_json_with_success(self, mock_session, make_mock_response):
        client = _create_client()
        url = urljoin(client.base_url, 'jobs/{}/result?'.format(JOB_ID))
        data = {
            'random': 'data'
        }
        response = make_mock_response(url=url, json_data=data)
        mock_session.request.return_value = response

        res = client._get_result_json(JOB_ID, {})

        assert res == data
        mock_session.request.assert_called_once_with(
            "GET",
            url,
            headers=client.default_headers)

    def test_get_result_json_with_kwargs_with_success(self, mock_session, make_mock_response):
        client = _create_client()
        kwarg_name_1 = 'name1'
        kwarg_value_1 = 1
        kwarg_name_2 = 'name2'
        kwarg_value_2 = 2
        if get_python_major_minor_version() < 35:
            url = urljoin(client.base_url,
                          'jobs/{0}/result?{1}={2}&{3}={4}'.format(JOB_ID,
                                                                   kwarg_name_2,
                                                                   kwarg_value_2,
                                                                   kwarg_name_1,
                                                                   kwarg_value_1))
        else:
            url = urljoin(client.base_url,
                          'jobs/{0}/result?{1}={2}&{3}={4}'.format(JOB_ID,
                                                                   kwarg_name_1,
                                                                   kwarg_value_1,
                                                                   kwarg_name_2,
                                                                   kwarg_value_2))
        data = {
            'random': 'data'
        }
        response = make_mock_response(url=url, json_data=data)
        mock_session.request.return_value = response

        res = client._get_result_json(JOB_ID, {kwarg_name_1: kwarg_value_1,
                                               kwarg_name_2: kwarg_value_2,
                                               'invisible': None})

        assert res == data
        mock_session.request.assert_called_once_with(
            "GET",
            Matcher(lambda x: '?name1=1&name2=2' in x or '?name2=2&name1=1' in x),
            headers=client.default_headers)

    def test_get_result_object_with_success(self, mock_session, make_mock_response):
        client = _create_client()
        url = urljoin(client.base_url, 'jobs/{}/result?'.format(JOB_ID))
        data = {
            'random': 'data'
        }
        response = make_mock_response(url=url, json_data=data)
        mock_session.request.return_value = response

        res = client._get_result_object(JOB_ID, {})

        assert res == data
        mock_session.request.assert_called_once_with(
            "GET",
            url,
            headers=client.default_headers)

    def test_get_result_object_with_kwargs_with_success(self, mock_session, make_mock_response):
        client = _create_client()
        kwarg_name_1 = 'name1'
        kwarg_value_1 = 1
        kwarg_name_2 = 'name2'
        kwarg_value_2 = 2
        if get_python_major_minor_version() < 35:
            url = urljoin(client.base_url,
                          'jobs/{0}/result?{1}={2}&{3}={4}'.format(JOB_ID,
                                                                   kwarg_name_2,
                                                                   kwarg_value_2,
                                                                   kwarg_name_1,
                                                                   kwarg_value_1))
        else:
            url = urljoin(client.base_url,
                          'jobs/{0}/result?{1}={2}&{3}={4}'.format(JOB_ID,
                                                                   kwarg_name_1,
                                                                   kwarg_value_1,
                                                                   kwarg_name_2,
                                                                   kwarg_value_2))
        data = {
            'random': 'data'
        }
        response = make_mock_response(url=url, json_data=data)
        mock_session.request.return_value = response

        res = client._get_result_object(JOB_ID, {kwarg_name_1: kwarg_value_1,
                                                 kwarg_name_2: kwarg_value_2,
                                                 'invisible': None})

        assert res == data
        mock_session.request.assert_called_once_with(
            "GET",
            Matcher(lambda x: '?name1=1&name2=2' in x or '?name2=2&name1=1' in x),
            headers=client.default_headers)


def _create_client():
    return GenericApiClient(TOKEN, API, VERSION, _pass_through_parse, _pass_through_parse)


def _pass_through_parse(x):
    return x


def get_python_major_minor_version():
    return sys.version_info.major * 10 + sys.version_info.minor



# -*- coding: utf-8 -*-
"""Job model"""
from .summarization_options import Summarization
from .job_status import JobStatus
from .translation_options import Translation


class Job:
    def __init__(
            self, id_, created_on, status,
            completed_on=None,
            name=None,
            callback_url=None,
            metadata=None,
            media_url=None,
            failure=None,
            failure_detail=None,
            duration_seconds=None,
            delete_after_seconds=None,
            skip_diarization=None,
            skip_punctuation=None,
            remove_disfluencies=None,
            filter_profanity=None,
            custom_vocabulary_id=None,
            speaker_channels_count=None,
            language=None,
            transcriber=None,
            verbatim=None,
            rush=None,
            segments_to_transcribe=None,
            remove_atmospherics=None,
            speakers_count=None,
            diarization_type=None,
            summarization: Summarization = None,
            translation: Translation = None):
        """
        :param id_: unique id of job
        :param created_on: date and time at which this job was started
        :param status: current job status 'IN_PROGRESS', 'TRANSCRIBED',
                       or 'FAILED'
        :param completed_on: date and time at which this job finished
                             being transcribed
        :param name: name of submitted file if local file was used
        :param callback_url: callback_url if provided
        :param metadata: metadata if provided
        :param media_url: url of transcribed media if job was submitted
                          this way
        :param failure: type of failure if job has failed
        :param failure_detail: more detailed failure message if job has failed
        :param duration_seconds: duration of submitted file in seconds
        :param delete_after_seconds: seconds before deletion if provided
        :param skip_diarization: whether to skip diarization if provided
        :param skip_punctuation: whether to skip punctuation if provided
        :param remove_disfluencies: whether to remove disfluencies if provided
        :param filter_profanity: whether to filter profanity if provided
        :param custom_vocabulary_id: custom vocabulary id if provided
        :param speaker_channels_count: speaker channels count if provided
        :param language: language of job
        :param transcriber: transcriber to use for job
        :param verbatim: whether to transcribe verbatim if provided for human transcription
        :param rush: whether to transcribe with rush if provided for human transcription
        :param segments_to_transcribe: segments to transcribe if provided for human transcription
        :param remove_atmospherics: Atmospherics such as <laugh>, <affirmative>, etc. will noT
            appear in the transcript.
        :param speakers_count: Use to specify the total number of unique speakers in the audio.
        :param diarization_type: Use to specify diarization type.
        """
        self.id = id_
        self.created_on = created_on
        self.status = status
        self.completed_on = completed_on
        self.name = name
        self.callback_url = callback_url,
        self.metadata = metadata
        self.media_url = media_url
        self.failure = failure
        self.failure_detail = failure_detail
        self.duration_seconds = duration_seconds
        self.delete_after_seconds = delete_after_seconds
        self.skip_diarization = skip_diarization
        self.skip_punctuation = skip_punctuation
        self.remove_disfluencies = remove_disfluencies
        self.filter_profanity = filter_profanity
        self.custom_vocabulary_id = custom_vocabulary_id
        self.speaker_channels_count = speaker_channels_count
        self.language = language
        self.transcriber = transcriber
        self.verbatim = verbatim
        self.rush = rush
        self.segments_to_transcribe = segments_to_transcribe
        self.remove_atmospherics = remove_atmospherics
        self.speakers_count = speakers_count
        self.diarization_type = diarization_type
        self.summarization = summarization
        self.translation = translation

    def __eq__(self, other):
        """Override default equality operator"""
        if isinstance(other, self.__class__):
            return self.__dict__ == other.__dict__
        return False

    @classmethod
    def from_json(cls, json):
        """Alternate constructor used for parsing json"""
        return cls(
            json['id'],
            json['created_on'],
            JobStatus.from_string(json['status']),
            completed_on=json.get('completed_on'),
            name=json.get('name'),
            callback_url=json.get('callback_url'),
            metadata=json.get('metadata'),
            media_url=json.get('media_url'),
            failure=json.get('failure'),
            failure_detail=json.get('failure_detail'),
            duration_seconds=json.get('duration_seconds'),
            delete_after_seconds=json.get('delete_after_seconds'),
            skip_diarization=json.get('skip_diarization'),
            skip_punctuation=json.get('skip_punctuation'),
            remove_disfluencies=json.get('remove_disfluencies'),
            filter_profanity=json.get('filter_profanity'),
            custom_vocabulary_id=json.get('custom_vocabulary_id'),
            speaker_channels_count=json.get('speaker_channels_count'),
            language=json.get('language'),
            transcriber=json.get('transcriber'),
            verbatim=json.get('verbatim'),
            rush=json.get('rush'),
            segments_to_transcribe=json.get('segments_to_transcribe'),
            remove_atmospherics=json.get('remove_atmospherics'),
            speakers_count=json.get('speakers_count'),
            diarization_type=json.get('diarization_type'),
            summarization=Summarization.from_json(json.get('summarization')),
            translation=Translation.from_json(json.get('translation'))
        )



# -*- coding: utf-8 -*-
"""Top-level package for rev_ai"""

__version__ = '2.21.0'

from .models import Job, JobStatus, Account, Transcript, Monologue, Element, MediaConfig, \
    CaptionType, GroupChannelsType, CustomVocabulary, TopicExtractionJob, TopicExtractionResult, \
    Topic, Informant, SpeakerName, LanguageIdentificationJob, LanguageIdentificationResult, \
    LanguageConfidence, SentimentAnalysisResult, SentimentValue, SentimentMessage, \
    SentimentAnalysisJob, CustomerUrlData, RevAiApiDeploymentConfigMap, RevAiApiDeployment



def get_error_test_cases(errors):
    return [ERRORS.get(e) for e in errors]


ERRORS = {
    'invalid-parameters': {
        "type": "https://www.rev.ai/api/v1/errors/invalid-parameters",
        "title": "Your request parameters did not validate",
        "status": 400
    },
    'unauthorized': {
        "title": "Authorization has been denied for this request",
        "status": 401
    },
    'forbidden': {
        "status": 403,
        "error": "The client is forbidden to submit that request"
    },
    'job-not-found': {
        "type": "https://www.rev.ai/api/v1/errors/job-not-found",
        "title": "could not find job",
        "status": 404
    },
    'invalid-job-state': {
        "allowed_values": [
            "transcribed"
        ],
        "current_value": "in_progress",
        "type": "https://rev.ai/api/v1/errors/invalid-job-state",
        "title": "Job is in invalid state",
        "detail": "Job is in invalid state to obtain the transcript",
        "status": 409
    }
}



# -*- coding: utf-8 -*-
"""Enum for group_channels_by types"""

from enum import Enum


class GroupChannelsType(str, Enum):
    SPEAKER = 'speaker'
    SENTENCE = 'sentence'
    WORD = 'word'

    @classmethod
    def from_string(cls, status):
        return cls[status.upper()]



# -*- coding: utf-8 -*-
"""Sentiment analysis result model"""

from .sentiment_value import SentimentValue


class SentimentAnalysisResult:
    def __init__(self, messages):
        """
        :param messages: list of sentimented statements from the input in order of how they appeared
                         in the input.
        """
        self.messages = messages

    def __eq__(self, other):
        """Override default equality operator"""
        if isinstance(other, self.__class__):
            return all(a == b for a, b in zip(self.messages, other.messages))
        return False

    @classmethod
    def from_json(cls, json):
        """Alternate constructor used for parsing json"""
        return cls([SentimentMessage.from_json(message) for message in json.get('messages', [])])


class SentimentMessage:
    def __init__(self, content, score, sentiment, timestamp=None, end_timestamp=None,
                 offset=None, length=None):
        """
        :param content: content of the informant, pulled from input
        :param score: Sentimental “score” of the content. Numbers less than 0 indicate a negative
                      (sad, angry) sentiment. Numbers above 0 indicate positive (joyful, happy)
                      sentiment
        :param: sentiment: Overall detected sentiment of the content, based off of score
        :param timestamp: time at which this element starts if input was json
        :param end_timestamp: time at which this element ends if input was json
        :param offset: Character index at which the content started in the source transcript,
                       excludes invisible characters
        :param length: Length of the content in characters, excludes invisible characters
        """
        self.content = content
        self.score = score
        self.sentiment = sentiment
        self.timestamp = timestamp
        self.end_timestamp = end_timestamp
        self.offset = offset
        self.length = length

    def __eq__(self, other):
        """Override default equality operator"""
        if isinstance(other, self.__class__):
            return self.__dict__ == other.__dict__
        return False

    @classmethod
    def from_json(cls, json):
        """Alternate constructor used for parsing json"""
        return cls(
            json['content'],
            json['score'],
            SentimentValue.from_string(json['sentiment']),
            json.get('ts'),
            json.get('end_ts'),
            json.get('offset'),
            json.get('length'))



# -*- coding: utf-8 -*-
"""Job model"""

from ...asynchronous.job_status import JobStatus


class TopicExtractionJob:
    def __init__(
            self, id_, created_on, status,
            completed_on=None,
            metadata=None,
            failure=None,
            failure_detail=None,
            word_count=None,
            delete_after_seconds=None):
        """
        :param id_: unique id of job
        :param created_on: date and time at which this job was started
        :param status: current job status 'IN_PROGRESS', 'COMPLETED',
                       or 'FAILED'
        :param completed_on: date and time at which this job finished
                             being processed
        :param metadata: metadata if provided
        :param failure: type of failure if job has failed
        :param failure_detail: more detailed failure message if job has failed
        :param word_count: count of words in job
        :param delete_after_seconds: seconds before deletion if provided
        """

        self.id = id_
        self.created_on = created_on
        self.status = status
        self.completed_on = completed_on
        self.metadata = metadata
        self.failure = failure
        self.failure_detail = failure_detail
        self.delete_after_seconds = delete_after_seconds
        self.word_count = word_count

    def __eq__(self, other):
        """Override default equality operator"""
        if isinstance(other, self.__class__):
            return self.__dict__ == other.__dict__
        return False

    @classmethod
    def from_json(cls, json):
        """Alternate constructor used for parsing json"""
        return cls(
            json['id'],
            json['created_on'],
            JobStatus.from_string(json['status']),
            completed_on=json.get('completed_on'),
            metadata=json.get('metadata'),
            failure=json.get('failure'),
            failure_detail=json.get('failure_detail'),
            word_count=json.get('word_count'),
            delete_after_seconds=json.get('delete_after_seconds'),
        )



# -*- coding: utf-8 -*-
"""Enum for possible sentiments"""

from enum import Enum


class SentimentValue(Enum):
    POSITIVE = 1
    NEGATIVE = 2
    NEUTRAL = 3

    def __str__(self):
        return self.name.lower()

    @classmethod
    def from_string(cls, status):
        return cls[status.upper()]



from .summarization_formatting_options import SummarizationFormattingOptions
from .summarization_job_status import SummarizationJobStatus
from .summarization_model import SummarizationModel


class SummarizationOptions:
    """Summarization request options."""

    def __init__(
            self,
            prompt: str = None,
            model: SummarizationModel = None,
            formatting_type: SummarizationFormattingOptions = None):
        self.prompt = prompt
        self.model = model
        self.type = formatting_type

    def to_dict(self):
        """Returns the raw form of the url data object as the api
        expects them"""
        dict_result = {}
        if self.prompt:
            dict_result['prompt'] = self.prompt
        if self.model:
            dict_result['model'] = self.model
        if self.type:
            dict_result['type'] = self.type

        return dict_result


class Summarization(SummarizationOptions):
    """Summarization options."""

    def __init__(
            self,
            prompt: str = None,
            model: SummarizationModel = None,
            formatting_type: SummarizationFormattingOptions = None,
            status: SummarizationJobStatus = None,
            completed_on: str = None,
            failure: str = None):
        super().__init__(prompt, model, formatting_type)
        self.status = status
        self.completed_on = completed_on
        self.failure = failure

    @classmethod
    def from_json(cls, json):
        if json is None:
            return None

        return cls(
            json.get('prompt'),
            json.get('model'),
            json.get('type'),
            json.get('status'),
            json.get('completed_on'),
            json.get('failure')
        )



# -*- coding: utf-8 -*-
from enum import Enum


class TranslationJobStatus(str, Enum):
    """Enum for Translation Job statuses"""
    IN_PROGRESS = "in_progress"
    FAILED = "failed"
    COMPLETED = "completed"



# -*- coding: utf-8 -*-
"""Enum for Job statuses"""

from enum import Enum


class JobStatus(str, Enum):
    IN_PROGRESS = 'in_progress'
    TRANSCRIBED = 'transcribed'
    FAILED = 'failed'
    COMPLETED = 'completed'

    @classmethod
    def from_string(cls, status):
        return cls[status.upper()]

    def __eq__(self, other):
        if isinstance(other, JobStatus):
            return self.value == other.value
        elif isinstance(other, str):
            return self.name.upper() == other.upper()
        else:
            return False



from typing import List, Dict
from .translation_language_options import TranslationLanguageOptions, TranslationLanguage


class TranslationOptions:
    """Translation request options."""
    def __init__(
            self,
            target_languages: List[TranslationLanguageOptions]):
        self.target_languages = target_languages

    def to_dict(self):
        """Returns the raw form of the url data object as the api
        expects them"""
        dict_result = {"target_languages": [tl.to_dict() for tl in self.target_languages]}

        return dict_result


class Translation(TranslationOptions):
    """Translation options."""
    def __init__(
            self,
            target_languages: List[TranslationLanguageOptions],
            completed_on: str = None):
        super().__init__(target_languages)
        self.completed_on = completed_on

    @classmethod
    def from_json(cls, json):
        if json is None:
            return None
        return cls(
            [TranslationLanguage.from_json(tl) for tl in json.get('target_languages')],
            json.get('completed_on')
        )



# -*- coding: utf-8 -*-
"""Unit tests for RevAiApiClient"""

import pytest
from src.rev_ai.apiclient import RevAiAPIClient
from src.rev_ai import __version__

TOKEN = "token"


class TestRevAiAPIClient:
    def test_constructor_with_success(self):
        client = RevAiAPIClient(TOKEN)

        headers = client.default_headers

        assert headers.get('User-Agent') == 'RevAi-PythonSDK/{}'.format(__version__)
        assert headers.get('Authorization') == 'Bearer {}'.format(TOKEN)

    @pytest.mark.parametrize('token', [None, ''])
    def test_constructor_with_no_token(self, token):
        with pytest.raises(ValueError, match='access_token must be provided'):
            RevAiAPIClient(token)



# -*- coding: utf-8 -*-
"""Insights Models"""

from .sentiment_analysis import SentimentAnalysisResult, SentimentValue, SentimentMessage, \
    SentimentAnalysisJob
from .topic_extraction import TopicExtractionResult, Topic, Informant, TopicExtractionJob



# -*- coding: utf-8 -*-
"""Unit tests for custom vocabulary"""
from src.rev_ai.custom_vocabularies_client import RevAiCustomVocabulariesClient
from src.rev_ai.models import CustomVocabulary

try:
    from urllib.parse import urljoin
except ImportError:
    from urlparse import urljoin

TOKEN = "token"
CV_ID = "cv1"
PHRASES = ["Patrick Henry Winston", "Noam Chomsky"]


class TestCustomVocabulary():

    def test_constructor(self):
        custom_vocabulary = CustomVocabulary(PHRASES)

        assert custom_vocabulary.phrases == PHRASES

    def test_no_aliasing_phrases_list(self):
        alias = [phrase for phrase in PHRASES]
        custom_vocabulary = CustomVocabulary(alias)

        alias.pop()

        assert len(custom_vocabulary.phrases) == 2
        assert custom_vocabulary.phrases == PHRASES

    def test_to_dict_method(self):
        custom_vocabulary = CustomVocabulary(PHRASES)

        custom_vocabulary_dict = custom_vocabulary.to_dict()

        assert custom_vocabulary_dict == {'phrases': PHRASES}


class TestCustomVocabularyEndpoints():
    def test_get_custom_vocabularies_information_success(self, mock_session, make_mock_response):
        data = {
            'id': CV_ID,
            'status': 'complete',
            'created_on': '2018-05-05T23:23:22.29Z'
        }
        client = RevAiCustomVocabulariesClient(TOKEN)
        url = urljoin(client.base_url, CV_ID)
        response = make_mock_response(url=url, json_data=data)
        mock_session.request.return_value = response

        res = client.get_custom_vocabularies_information(CV_ID)

        assert res == data
        mock_session.request.assert_called_once_with("GET", url,
                                                     headers=client.default_headers)

    def test_get_list_of_custom_vocabularies_success(self, mock_session, make_mock_response):
        data = {
            'id': CV_ID,
            'status': 'complete',
            'created_on': '2018-05-05T23:23:22.29Z'
        }
        client = RevAiCustomVocabulariesClient(TOKEN)
        response = make_mock_response(url=client.base_url, json_data=[data])
        mock_session.request.return_value = response

        res = client.get_list_of_custom_vocabularies()

        assert res == [data]
        mock_session.request.assert_called_once_with("GET", client.base_url,
                                                     headers=client.default_headers)

    def test_get_list_of_custom_vocabularies_with_limit(self, mock_session, make_mock_response):
        data = {
            'id': CV_ID,
            'status': 'complete',
            'created_on': '2018-05-05T23:23:22.29Z'
        }
        limit = 5
        client = RevAiCustomVocabulariesClient(TOKEN)
        url = '{}?limit={}'.format(client.base_url, limit)
        response = make_mock_response(url=client.base_url, json_data=[data])
        mock_session.request.return_value = response

        res = client.get_list_of_custom_vocabularies(limit)

        assert res == [data]
        mock_session.request.assert_called_once_with("GET", url,
                                                     headers=client.default_headers)

    def test_delete_custom_vocabulary_success(self, mock_session, make_mock_response):
        client = RevAiCustomVocabulariesClient(TOKEN)
        url = urljoin(client.base_url, CV_ID)
        response = make_mock_response(url=url, status=204)
        mock_session.request.return_value = response

        res = client.delete_custom_vocabulary(CV_ID)

        assert res is None
        mock_session.request.assert_called_once_with("DELETE", url,
                                                     headers=client.default_headers)



import json

import pytest

from src.rev_ai.apiclient import RevAiAPIClient
from src.rev_ai.models import RevAiApiDeploymentConfigMap, RevAiApiDeployment
from src.rev_ai.models.asynchronous.summarization_formatting_options import SummarizationFormattingOptions
from src.rev_ai.models.asynchronous.summarization_job_status import SummarizationJobStatus
from src.rev_ai.models.asynchronous.summarization_options import SummarizationOptions
from src.rev_ai.models.asynchronous.summarization_model import SummarizationModel

try:
    from urllib.parse import urljoin
except ImportError:
    from urlparse import urljoin

TOKEN = "token"
JOB_ID = '1'
SPEECH_TO_TEXT_URL = f"{RevAiApiDeploymentConfigMap[RevAiApiDeployment.US]['base_url']}/speechtotext/v1/"
JOB_ID_URL = urljoin(SPEECH_TO_TEXT_URL, 'jobs/{}'.format(JOB_ID))
JOBS_URL = urljoin(SPEECH_TO_TEXT_URL, 'jobs')
JOB_TRANSCRIPT_SUMMARY_URL = '{}/transcript/summary'.format(JOB_ID_URL)


@pytest.mark.usefixtures('mock_session', 'make_mock_response')
class TestAsyncSummarization():

    def test_submit_local_file(self, mocker, mock_session, make_mock_response):
        status = 'transcribed'
        created_on = '2018-05-05T23:23:22.29Z'
        completed_on = '2018-05-05T23:23:22.30Z'
        data = {
            'id': JOB_ID,
            'created_on': created_on,
            'status': status,
            'summarization': {
                'prompt': 'Try to summarize this transcript as good as you possibly can',
                'model': 'premium',
                'type': 'bullets',
                'status': 'completed',
                'completed_on': completed_on
            }
        }
        response = make_mock_response(url=JOB_ID_URL, json_data=data)
        mock_session.request.return_value = response
        client = RevAiAPIClient(TOKEN)

        with mocker.patch('src.rev_ai.apiclient.open', create=True)() as file:
            job = client.submit_job_local_file('test_mp3.mp3',
                                               language="en",
                                               summarization_config=SummarizationOptions(
                                                   prompt="Try to summarize this transcript as good as you possibly can",
                                                   model=SummarizationModel.PREMIUM,
                                                   formatting_type=SummarizationFormattingOptions.BULLETS

                                               ))
        mock_session.request.assert_called_once_with(
            "POST",
            JOBS_URL,
            files={
                'media': ('test_mp3.mp3', file),
                'options': (
                    None,
                    json.dumps({
                        'language': 'en',
                        'summarization_config': {
                            'prompt': "Try to summarize this transcript as good as you possibly can",
                            'model': 'premium',
                            'type': 'bullets'
                        }
                    }, sort_keys=True)
                )
            },
            headers=client.default_headers
        )

        assert job.summarization is not None
        assert job.summarization.model == SummarizationModel.PREMIUM
        assert job.summarization.type == SummarizationFormattingOptions.BULLETS
        assert job.summarization.prompt == "Try to summarize this transcript as good as you possibly can"

    def test_submit_source_url(self, mock_session, make_mock_response):
        status = 'transcribed'
        created_on = '2018-05-05T23:23:22.29Z'
        completed_on = '2018-05-05T23:23:22.30Z'
        data = {
            'id': JOB_ID,
            'status': status,
            'created_on': created_on,
            'summarization': {
                'prompt': 'Try to summarize this transcript as good as you possibly can',
                'model': 'premium',
                'type': 'bullets',
                'status': 'completed',
                'completed_on': completed_on
            }
        }
        response = make_mock_response(url=JOB_ID_URL, json_data=data)
        mock_session.request.return_value = response

        client = RevAiAPIClient(TOKEN)

        job = client.submit_job_url('https://example.com/test.mp3',
                                    language="en",
                                    summarization_config=SummarizationOptions(
                                        "Try to summarize this transcript as good as you possibly can",
                                        SummarizationModel.PREMIUM,
                                        SummarizationFormattingOptions.BULLETS

                                    ))
        mock_session.request.assert_called_once_with(
            "POST",
            JOBS_URL,
            json={
                'media_url': 'https://example.com/test.mp3',
                'language': 'en',
                'summarization_config': {
                    'prompt': "Try to summarize this transcript as good as you possibly can",
                    'model': 'premium',
                    'type': 'bullets'
                }
            },
            headers=client.default_headers
        )

        assert job.summarization is not None
        assert job.summarization.status == SummarizationJobStatus.COMPLETED
        assert job.summarization.model == SummarizationModel.PREMIUM
        assert job.summarization.type == SummarizationFormattingOptions.BULLETS
        assert job.summarization.prompt == "Try to summarize this transcript as good as you possibly can"

    def test_get_transcript_summary_text(self, mock_session, make_mock_response):
        url = JOB_TRANSCRIPT_SUMMARY_URL
        client = RevAiAPIClient(TOKEN)
        response = make_mock_response(url=url, text='transcript summary')
        mock_session.request.return_value = response

        summary = client.get_transcript_summary_text(JOB_ID)
        assert summary == 'transcript summary'
        mock_session.request.assert_called_once_with(
            "GET",
            url,
            headers=self.hdr_fixture(client, {'Accept': 'text/plain'})
        )

    def test_get_transcript_summary_object(self, mock_session, make_mock_response):
        url = JOB_TRANSCRIPT_SUMMARY_URL
        client = RevAiAPIClient(TOKEN)
        data = {
            'summary': 'transcript summary'
        }
        response = make_mock_response(url=url, json_data=data)
        mock_session.request.return_value = response

        summary = client.get_transcript_summary_object(JOB_ID)
        assert summary.summary == 'transcript summary'
        mock_session.request.assert_called_once_with(
            "GET",
            url,
            headers=self.hdr_fixture(client, {'Accept': 'application/json'})
        )

    def test_get_transcript_summary_object_paragraph(self, mock_session, make_mock_response):
        url = JOB_TRANSCRIPT_SUMMARY_URL
        data = {
            'summary': 'transcript summary'
        }
        client = RevAiAPIClient(TOKEN)
        response = make_mock_response(url=url, json_data=data)
        mock_session.request.return_value = response

        summary = client.get_transcript_summary_object(JOB_ID)
        assert summary.summary == 'transcript summary'
        mock_session.request.assert_called_once_with(
            "GET",
            url,
            headers=self.hdr_fixture(client, {'Accept': 'application/json'})
        )

    def test_get_transcript_summary_json_paragraph_as_stream(self, mock_session, make_mock_response):
        url = JOB_TRANSCRIPT_SUMMARY_URL
        data = {
            "summary": "transcript summary"
        }

        client = RevAiAPIClient(TOKEN)
        response = make_mock_response(url=url, json_data=data)
        mock_session.request.return_value = response

        summary = client.get_transcript_summary_json_as_stream(JOB_ID)

        s = summary.content.decode('utf-8')
        s = s.replace('\'', '"')

        assert json.loads(s) == data

        mock_session.request.assert_called_once_with(
            "GET",
            url,
            headers=self.hdr_fixture(client, {'Accept': 'application/json'}),
            stream=True
        )

    def test_get_transcript_summary_json_bullets(self, mock_session, make_mock_response):
        url = JOB_TRANSCRIPT_SUMMARY_URL
        data = {
            'bullet_points': ['bullet1', 'bullet2']
        }
        client = RevAiAPIClient(TOKEN)
        response = make_mock_response(url=url, json_data=data)
        mock_session.request.return_value = response

        summary_json = client.get_transcript_summary_json(JOB_ID)
        assert summary_json is not None
        assert summary_json['bullet_points'] is not None

        mock_session.request.assert_called_once_with(
            "GET",
            url,
            headers=self.hdr_fixture(client, {'Accept': 'application/json'})
        )

    @staticmethod
    def hdr_fixture(client: RevAiAPIClient, additional_headers):
        hdr = {}
        hdr.update(client.default_headers)
        hdr.update(additional_headers)
        return hdr



import pytest
from src.rev_ai import __version__
from src.rev_ai.custom_vocabularies_client import RevAiCustomVocabulariesClient

try:
    from unittest import mock
except ImportError:
    import mock

TOKEN = "token"


class TestCustomVocabulariesClient:
    def test_constructor_with_success(self):
        client = RevAiCustomVocabulariesClient(TOKEN)

        headers = client.default_headers

        assert headers.get(
            'User-Agent') == 'RevAi-PythonSDK/{}'.format(__version__)
        assert headers.get('Authorization') == 'Bearer token'
        assert client.base_url == 'https://api.rev.ai/speechtotext/v1/vocabularies/'

    @pytest.mark.parametrize('token', [None, ''])
    def test_constructor_with_no_token(self, token):
        with pytest.raises(ValueError, match='access_token must be provided'):
            RevAiCustomVocabulariesClient(token)



# -*- coding: utf-8 -*-
"""Speech recognition tools for using Rev AI"""

import requests
from requests.exceptions import HTTPError
from . import __version__
from . import CustomVocabulary


class BaseClient:
    """Base for client's making HTTP Requests to Rev AI Apis"""

    def __init__(self, access_token):
        """Constructor

        :param access_token: access token which authorizes all requests and
                             links them to your account. Generated on the
                             settings page of your account dashboard
                             on Rev AI
        """
        if not access_token:
            raise ValueError('access_token must be provided')

        self.default_headers = {
            'Authorization': 'Bearer {}'.format(access_token),
            'User-Agent': 'RevAi-PythonSDK/{}'.format(__version__)
        }

    def _make_http_request(self, method, url, **kwargs):
        """Wrapper method for initiating HTTP requests and handling potential
            errors.

        :param method: string of HTTP method request
        :param url: string containing the URL to make the request to
        :param (optional) **kwargs: potential extra arguments including header
            and stream
        :raises: HTTPError
        """
        headers = self.default_headers.copy()
        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
            del kwargs['headers']
        with requests.Session() as session:
            response = session.request(method, url, headers=headers, **kwargs)

        try:
            response.raise_for_status()
            return response
        except HTTPError as err:
            if (response.content):
                err.args = (err.args[0] +
                            "; Server Response : {}".
                            format(response.content.decode('utf-8')),)
            raise



from typing import List, Dict


class Summary:
    """Transcript summary model."""
    def __init__(
            self,
            summary: str,
            bullet_points: List[str]):
        self.summary = summary
        self.bullet_points = bullet_points

    @classmethod
    def from_json(cls, json):
        return cls(
            json.get('summary'),
            json.get('bullet_points')
        )



from src.rev_ai import JobStatus


class TestJobStatus:
    def test_job_status_equality(self):
        assert JobStatus.IN_PROGRESS == JobStatus.IN_PROGRESS
        assert JobStatus.IN_PROGRESS == 'IN_PROGRESS'
        assert JobStatus.IN_PROGRESS == 'in_progress'
        assert JobStatus.IN_PROGRESS.name == 'IN_PROGRESS'
        assert JobStatus.from_string('in_progress') == JobStatus.IN_PROGRESS
        assert JobStatus.from_string('IN_PROGRESS') == JobStatus.IN_PROGRESS

        assert JobStatus.TRANSCRIBED == JobStatus.TRANSCRIBED
        assert JobStatus.TRANSCRIBED == 'TRANSCRIBED'
        assert JobStatus.TRANSCRIBED == 'transcribed'
        assert JobStatus.TRANSCRIBED.name == 'TRANSCRIBED'
        assert JobStatus.from_string('transcribed') == JobStatus.TRANSCRIBED
        assert JobStatus.from_string('TRANSCRIBED') == JobStatus.TRANSCRIBED

        assert JobStatus.FAILED == JobStatus.FAILED
        assert JobStatus.FAILED == 'FAILED'
        assert JobStatus.FAILED == 'failed'
        assert JobStatus.FAILED.name == 'FAILED'
        assert JobStatus.from_string('failed') == JobStatus.FAILED
        assert JobStatus.from_string('FAILED') == JobStatus.FAILED

        assert JobStatus.COMPLETED == JobStatus.COMPLETED
        assert JobStatus.COMPLETED == 'COMPLETED'
        assert JobStatus.COMPLETED == 'completed'
        assert JobStatus.COMPLETED.name == 'COMPLETED'
        assert JobStatus.from_string('completed') == JobStatus.COMPLETED
        assert JobStatus.from_string('COMPLETED') == JobStatus.COMPLETED



# -*- coding: utf-8 -*-
"""Language identification result model"""


class LanguageIdentificationResult:
    def __init__(self, top_language, language_confidences):
        """
        :param top_language: Language code of predicted language
        :param language_confidences: List of all potential languages with their corresponding
            confidence scores
        """
        self.top_language = top_language
        self.language_confidences = language_confidences

    def __eq__(self, other):
        """Override default equality operator"""
        if isinstance(other, self.__class__):
            return all(a == b for a, b in
                       zip(self.language_confidences, other.language_confidences)) \
                and self.top_language == other.top_language
        return False

    @classmethod
    def from_json(cls, json):
        """Alternate constructor used for parsing json"""
        return cls(
            json['top_language'],
            [LanguageConfidence.from_json(language_confidence) for
             language_confidence in json.get('language_confidences', [])])


class LanguageConfidence:
    def __init__(self, language, confidence):
        """
        :param language: Language code of predicted language
        :param confidence: Confidence score of the predicted language, ranges from 0.00 to 1.00
        """
        self.language = language
        self.confidence = confidence

    def __eq__(self, other):
        """Override default equality operator"""
        if isinstance(other, self.__class__):
            return self.language == other.language and self.confidence == other.confidence
        return False

    @classmethod
    def from_json(cls, json):
        """Alternate constructor used for parsing json"""
        return cls(
            json['language'],
            json['confidence'])
